"""
.. module:: __init__.py

__init__.py
*************

:Description: __init__.py

    

:Authors: bejar
    

:Version: 

:Created on: 13/03/2015 16:13 

"""

__author__ = 'bejar'

from .Discretizer import Discretizer

__all__ = ['Discretizer']